package com.huasheng.quant.request;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Sets;
import com.huasheng.quant.constant.ISystemError;
import com.huasheng.quant.exception.BuzzException;
import com.huasheng.quant.utils.NetworkUtil;
import com.huasheng.quant.utils.RsaSignature;
import com.huasheng.quant.vo.HttpResult;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.NetworkInterface;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Set;
import java.util.TreeSet;

public class DeviceBindRequst {

    private static final Logger logger = LoggerFactory.getLogger(DeviceBindRequst.class);

    private static String testHttpUrl = "https://openapi-daily.hstong.com";
    private static String onlineHttpUrl = "https://openapi.hstong.com";

    public static HttpResult sendAuthCode(String countryCode, String mobile, String pk, String env) {

        String sign = RsaSignature.signStr(mobile, pk);
        String response = httpSendAuthCode(getDomain(env) + "/hs/devicebind/sendAuthCode", countryCode, mobile, sign);
        System.out.println("\r\n验证码已经发送请注意查收:" + mobile + "  ...............服务器响应:" + response);
        HttpResult httpResult = covertHttpResult(response);
        return httpResult;
    }

    public static HttpResult sendBindRequest(String countryCode, String mobile, String code, String pk, String env) {
        TreeSet<String> set = Sets.newTreeSet();
        Set<NetworkInterface> networkInterfaces = NetworkUtil.getPhysicalNICs();
        if (networkInterfaces == null || networkInterfaces.size() == 0) {
            networkInterfaces = NetworkUtil.getNICs(NetworkUtil.Filter.ALL);
            System.out.println("\r\n获取物理设备失败......");
        }
        networkInterfaces.forEach((networkInterface) -> {
            String macAddress = NetworkUtil.getMacAddress(networkInterface, "-");
            if (StringUtils.isNotBlank(macAddress)) {
                set.add(macAddress);
            }
        });
        if (set.size() == 0) {
            System.out.println("\r\n获取绑定设备失败");
            HttpResult httpResult = new HttpResult();
            httpResult.setRespCode(ISystemError.GET_BIND_DEVICE_ERROR.getCode());
            httpResult.setRespMsg(ISystemError.GET_BIND_DEVICE_ERROR.getDescription());
            return httpResult;
        }
        String deviceNo = set.first();
        String sign = RsaSignature.signStr(mobile, pk);
        String response = httpBind(getDomain(env) + "/hs/devicebind/bind", countryCode, mobile, code, deviceNo, sign);
        System.out.println("\r\n开始设备绑定:" + mobile + "  设备号:" + set.first() + "  服务器响应:" + response);
        HttpResult httpResult = covertHttpResult(response);
        return httpResult;
    }


    public static String httpSendAuthCode(String httpUrl, String countryCode, String mobile, String token) throws RuntimeException {
        StringBuffer body = new StringBuffer();
        body.append("countryCode=").append(countryCode);
        body.append("&mobile=").append(mobile);
        body.append("&sign=").append(token);
        String response = postRequest(httpUrl, body);
        return response;
    }

    public static String httpBind(String httpUrl, String countryCode, String mobile, String code, String deviceNo, String token) throws RuntimeException {
        StringBuffer body = new StringBuffer();
        body.append("code=").append(code);
        body.append("&countryCode=").append(countryCode);
        body.append("&mobile=").append(mobile);
        body.append("&deviceNo=").append(deviceNo);
        body.append("&deviceType=").append(System.getProperty("os.name"));
        body.append("&deviceName=").append("open-api");
        body.append("&sign=").append(token);
        String response = postRequest(httpUrl, body);
        return response;
    }

    private static String postRequest(String httpUrl, StringBuffer body) throws RuntimeException {
        HttpURLConnection conn = null;
        Writer writer = null;
        Reader reader = null;
        try {
            URL url = new URL(httpUrl);
            conn = (HttpURLConnection) url.openConnection();
            conn.setRequestMethod("POST");
            conn.setDoInput(true);
            conn.setDoOutput(true);
            conn.setConnectTimeout(15000);
            conn.setReadTimeout(15000);
            conn.setUseCaches(false);
            conn.connect();

            writer = new OutputStreamWriter(conn.getOutputStream(), StandardCharsets.UTF_8);
            writer.write(body.toString());
            writer.flush();

            int responseCode = conn.getResponseCode();
            if (responseCode == HttpURLConnection.HTTP_OK) {
                reader = new InputStreamReader(conn.getInputStream(), StandardCharsets.UTF_8);
                StringBuilder response = new StringBuilder();
                final char[] buff = new char[1024];
                int read = 0;
                while ((read = reader.read(buff)) > 0) {
                    response.append(buff, 0, read);
                }
                return response.toString();
            } else {
                System.out.println("响应状态码-------------------" + String.valueOf(conn.getResponseCode()));
                throw new BuzzException(String.valueOf(conn.getResponseCode()), conn.getResponseMessage());
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        } finally {
            try {
                if (writer != null) {
                    writer.close();
                }
                if (reader != null) {
                    reader.close();
                }
                if (conn != null) {
                    conn.disconnect();
                }
            } catch (Exception e) {
                logger.error(e.getMessage(), e);
            }
        }
    }

    private static String getDomain(String env) {
        if ("1".equals(env)) {
            return testHttpUrl;
        } else {
            return onlineHttpUrl;
        }

    }

    public static HttpResult covertHttpResult(String response) {
        HttpResult errorResult = new HttpResult();
        errorResult.setRespCode(ISystemError.UNKNOWN_ERROR.getCode());
        errorResult.setRespMsg(ISystemError.UNKNOWN_ERROR.getDescription());
        if (StringUtils.isNotBlank(response)) {
            try {
                return JSONObject.parseObject(response, HttpResult.class);
            } catch (Exception e) {
                return errorResult;
            }
        } else {
            return errorResult;
        }

    }

}
