package com.huasheng.quant.utils;

import org.apache.commons.codec.binary.Hex;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.charset.StandardCharsets;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

public class RsaSignature {

	private static final Logger log = LoggerFactory.getLogger(RsaSignature.class);

	public static final String SIGN_ALGORITHMS = "SHA1WithRSA";

	/**
	 * RSA私钥签名
	 */
	public static byte[] sign(byte[] content, String privateKeyByBase64) {
		if (content == null || content.length == 0 || StringUtils.isBlank(privateKeyByBase64)) {
			return null;
		}
		try {
			PKCS8EncodedKeySpec priPKCS8 = new PKCS8EncodedKeySpec(Base64.decode(privateKeyByBase64));
			KeyFactory keyf = KeyFactory.getInstance("RSA");
			PrivateKey priKey = keyf.generatePrivate(priPKCS8);
			Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
			signature.initSign(priKey);
			signature.update(content);
			return signature.sign();
		} catch (Exception e) {
			log.error("签名异常", e);
		}

		return null;
	}

	public static String signStr(String content, String privateKey) {
		if (StringUtils.isBlank(content) || StringUtils.isBlank(privateKey)) {
			return null;
		}
		byte[] bytes = content.getBytes(StandardCharsets.UTF_8);
		byte[] signBytes = sign(bytes, privateKey);
		if (signBytes == null || signBytes.length == 0) {
			throw new RuntimeException("获取数据签名发生异常");
		}
		return Hex.encodeHexString(signBytes);
	}
	
}
